@@ -74,6 +74,11 @@ static const char* username = "whoopsie";
  *  - The MAC address of the first non-loopback device, SHA-512 hashed */
 static char* whoopsie_identifier = NULL;
 
+/*  Whether or not to verify the ssl peer when submitting crashes. Either:
+ *  - 0 verification will NOT occur
+ *  - 1 verification will occur */
+static long verifypeer = 1;
+
 /* The URL for sending the initial crash report */
 static char* crash_db_submit_url = NULL;
 
@@ -164,6 +169,7 @@ static const char* acceptable_fields[] = {
     "DpkgHistoryLog.txt",
     "DpkgTerminalLog.txt",
     "AptDaemon",
+    "HWEunsupported",
     /* aptdaemon */
     "AptConfig",
     "SourcesList",
@@ -173,6 +179,8 @@ static const char* acceptable_fields[] = {
     "SysLog",
     /* openjdk-8 */
     "HotspotError",
+    /* desktop app debugging info */
+    "JournalErrors",
 
     NULL,
 };
@@ -319,6 +327,7 @@ upload_report (const char* message_data, uint32_t message_len, struct response_s
     curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, server_response);
     curl_easy_setopt (curl, CURLOPT_WRITEDATA, s);
     curl_easy_setopt (curl, CURLOPT_VERBOSE, 0L);
+    curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, verifypeer);
 
     result_code = curl_easy_perform (curl);
     curl_slist_free_all(list);
@@ -466,19 +475,19 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
                                  0, "Report value too long.");
                     goto error;
                 }
+                g_hash_table_steal (hash_table, key);
                 value = g_realloc (value, value_pos + 1 + value_length + 1);
                 value_p = value + value_pos;
                 *value_p = '\n';
                 value_p++;
-                g_hash_table_steal (hash_table, key);
             } else {
-                value = g_realloc (value, value_length + 1);
-                value_p = value;
                 /* Make sure we properly free the old empty string value */
                 old_value = g_hash_table_lookup (hash_table, key);
                 if (old_value)
                     g_free (old_value);
                 g_hash_table_steal (hash_table, key);
+                value = g_realloc (value, value_length + 1);
+                value_p = value;
             }
             memcpy (value_p, p, value_length);
             value_p[value_length] = '\0';
@@ -487,6 +496,12 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
                 if (*c != '\t' && *c >= '\0' && *c < ' ')
                     *c = '?';
             value_p += value_length;
+            if (g_hash_table_contains (hash_table, key) == TRUE) {
+                g_set_error (error,
+                    g_quark_from_static_string ("whoopsie-quark"),
+                    0, "Report key must not be a duplicate.");
+                goto error;
+            }
             g_hash_table_insert (hash_table, key, value ? value : g_strdup(""));
             p = token_p + 1;
         } else {
@@ -557,6 +572,12 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
             }
             p = token_p + 1;
 
+            if (g_hash_table_contains (hash_table, key) == TRUE) {
+                g_set_error (error,
+                    g_quark_from_static_string ("whoopsie-quark"),
+                    0, "Report key must not be a duplicate.");
+                goto error;
+            }
             g_hash_table_insert (hash_table, key, value ? value : g_strdup(""));
         }
     }
@@ -631,6 +652,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, server_response);
     curl_easy_setopt (curl, CURLOPT_WRITEDATA, &s);
     curl_easy_setopt (curl, CURLOPT_VERBOSE, 0L);
+    curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, verifypeer);
 
     result_code = curl_easy_perform (curl);
     curl_slist_free_all(list);
@@ -778,9 +800,10 @@ process_existing_files (const char* report_dir)
             g_free (crash_file);
             continue;
         } else if (online_state && parse_and_upload_report (crash_file)) {
-            if (!mark_handled (crash_file))
+            if (!mark_handled (crash_file)) {
                 log_msg ("Unable to mark report as seen (%s) removing it.\n", crash_file);
                 g_unlink (crash_file);
+            }
         }
 
         g_free (upload_file);
@@ -1048,6 +1071,12 @@ main (int argc, char** argv)
     else
         whoopsie_identifier_generate (&whoopsie_identifier, &err);
 
+    env = g_getenv ("CRASH_DB_NOVERIFYPEER");
+    if (env != NULL) {
+        log_msg ("CRASH_DB_NOVERIFYPEER is set, not verifying ssl cert.\n");
+        verifypeer = 0;
+    }
+
     if (err) {
         log_msg ("%s\n", err->message);
         g_error_free (err);
